unit MemMK;
{
              :
   +---------------------+---------------------------+
   |         |                   |
   +---------------------+---------------------------+
   |         0           |           BASIC           |
   | MemTotal - DEFM * 8 |  ,  Z |
   |      MemTotal       |       Z..A      |
   |   MemTotal + 208    |         $       |
   |   MemTotal + 239    |     96  |
   |   MemTotal + 246    |        |
   +---------------------+---------------------------+

                   
   +----------+--------+-------+------------------------+
   |  |  |    |                |
   +----------+--------+-------+------------------------+
   |    0     |   4    | ENDP0 |      P0      |
   |   ...    |  ...   |  ...  |    FFFFFFFF-      |
   |    36    |   4    | ENDP9 |      P9      |
   |    40    |   4    | DEFM  |    |
   +----------+--------+-------+------------------------+
}
interface

uses SysUtils;

type SmallStr = string[7];
     BigStr = string[30];

var MemTotal: Integer; {  }

  procedure MemCreate;
  procedure MemDestroy;
  procedure MemSave (FileName: string);
  procedure MemLoad (FileName: string);  
  function GetStrVar (N: Integer): SmallStr;
  procedure SetStrVar (N: Integer; Val: SmallStr);
  function GetBigStr: BigStr;
  procedure SetBigVar (Val: BigStr);

implementation

const HiMem = 290; {-    }

var Mem: PByteArray;
    MemInd: Integer = 0;

procedure MemDestroy;
//   :
begin
  if MemInd > 0 then FreeMem (Mem, MemInd);
end; {proc MemDestroy}

procedure ErrMes (Mes: string);
//   :
begin
//  Beep;

end; {proc ErrMes}

function Peek (Addr: Integer): Byte;
//  :
begin
  if (Addr >= 0) and (Addr < MemInd)
    then Result := Mem[Addr]
    else Result := 0;
end; {proc Peek}

procedure Poke (Addr: Integer; Val: Byte);
//   :
begin
  if (Addr >= 0) and (Addr < MemInd) then Mem[Addr] := Val;
end; {proc Poke}

function GetStrVar (N: Integer): SmallStr;
//  ,  :
var Adr: Integer;
    I: Integer;
begin
  Adr := MemTotal + 208 - N * 8;
  I := 1;
  Result := '';
  while (I < 8) and (Peek (Adr) <> 0) do
  begin
    Result := Concat (Result, Chr (Peek (Adr)));
    Inc (I);
    Dec (Adr);
  end; {while}
end; {proc GetStrVar}

procedure SetStrVar (N: Integer; Val: SmallStr);
//    :
var Adr: Integer;
    I: Integer;
begin
  Adr := MemTotal + 208 - N * 8;
  for I := 1 to Length (Val) do
  begin
    Poke (Adr, Ord (Val[I]));
    Dec (Adr);
  end; {for}
  Poke (Adr, 0);
end; {proc GetStrVar}

function GetBigStr: BigStr;
//   :
var Adr: Integer;
begin
  Adr := MemTotal + 238;
  Result := '';
  while Peek (Adr) <> 0 do
  begin
    Result := Concat (Result, Chr (Peek (Adr)));
    Dec (Adr);
  end; {while}
end; {proc GetStrVar}

procedure SetBigVar (Val: BigStr);
//   :
var Adr: Integer;
    I: Integer;
begin
  Adr := MemTotal + 238;
  for I := 1 to Length (Val) do
  begin
    Poke (Adr, Ord (Val[I]));
    Dec (Adr);
  end; {for}
  Poke (Adr, 0);
end; {proc GetStrVar}

procedure MemCreate;
//    :
begin
  MemInd := MemTotal + HiMem;
  GetMem (Mem, MemInd);

end; {proc MemCreate}

procedure MemSave (FileName: string);
//   :
var F: file of Byte;
    I: Integer;
begin
  {$I-}
  AssignFile (F, FileName);
  ReWrite (F);
  for I := 0 to MemInd - 1 do
    Write (F, Mem[I]);
  CloseFile (F);
  {$I+}
  if IOResult <> 0 then ErrMes (Concat ('     "', FileName, '".'));
end; {proc MemSave}

procedure MemLoad (FileName: string);
//   :
var F: file of Byte;
    I, OldTotal: Integer;
    B: Boolean;
begin
  B := True;
  OldTotal := MemTotal;
  if FileExists (FileName) then
  begin
    {$I-}
    AssignFile (F, FileName);
    MemTotal := FileSize (F) - HiMem;
    MemCreate;
    Reset (F);
    for I := 0 to MemTotal + HiMem do
      Read (F, Mem[I]);
    CloseFile (F);
    {$I+}
    if IOResult <> 0
      then ErrMes (Concat ('     "', FileName, '".'))
      else B := True;
  end; {if}
  if B then
  begin
    MemTotal := OldTotal;
    MemCreate;
  end; {if}
end; {proc MemSave}


end.
